/* cmParse - parse clone map and turn it into
 * format our job maker can handle. */
#include "common.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include "obscure.h"
#include "linefile.h"
#include "portable.h"
#include "errabort.h"
#include "cmBac.h"
#include "hash.h"

char *finDir = "/projects/compbio/experiments/hg/gs.1/fin/fa";
char *unfinDir = "/projects/compbio/experiments/hg/gs.1/unfin/fa";
char *predraftDir = "/projects/compbio/experiments/hg/gs.1/predraft/fa";

FILE *errLog;

void usage()
/* Describe usage and exit. */
{
errAbort(
   "cmParse - turn clone map into more easily processed\n"
   "form\n"
   "usage:\n"
   "    cmParse input outDir\n");
}

struct cmChrom
/* Clone map info for one chromosome. */
    {
    struct cmChrom *next;
    char *name;
    struct cmContig *orderedList;
    struct cmContig *randomList;
    };

struct cmContig
/* Clone map info for one contig (possibly with gaps). */
    {
    struct cmContig *next;
    char *name;
    struct cmMini *miniList;
    };

struct cmMini
/* Clone map info for a part of contig with no gaps. */
    {
    struct cmMini *next;
    int ix;
    struct cmBac *bacList;
    };

boolean inContig(struct cmContig *contig, struct cmBac *bac)
/* Return TRUE if bac already in contig. */
{
struct cmMini *mini;
for (mini = contig->miniList; mini != NULL; mini = mini->next)
    {
    struct cmBac *b;
    for (b = mini->bacList; b != NULL; b = b->next)
	{
	if (sameString(b->acc, bac->acc))
	    {
	    fprintf(errLog, "%s duplicate %s\n", contig->name, bac->acc);
	    return TRUE;
	    }
	}
    }
return FALSE;
}


void readContigList(struct lineFile *in, struct cmContig **pContigList)
/* Read in contigs. */
{
boolean gotEnd = FALSE;
char *lastContig = "";
int lineSize;
char *line;
char *words[16];
int wordCount;
struct cmContig *contig;
struct cmMini *mini = NULL;
struct cmBac *bac;
int miniIx = 0;

while (lineFileNext(in, &line, &lineSize))
    {
    if (startsWith("end human", line))
	{
	gotEnd = TRUE;
	break;
	}
    wordCount = chopLine(line, words);
    if (words[0][0] == '@' || words[0][0] == '*')
	continue;	
    if (sameWord(words[0], "COMMITTED"))
	{
	mini = NULL;
	}
    else
	{
	if (wordCount == 5)
	    {
	    words[5] = words[6] = "0";
	    wordCount = 7;
	    }
	if (wordCount != 7)
	    errAbort("Expecting 5 or 7 words line %d of %s", in->lineIx, in->fileName);
	bac = cmBacLoad(words);
	bac->pos *= 5;
	if (!sameString(bac->contig, lastContig))
	    {
	    lastContig = bac->contig;
	    AllocVar(contig);
	    contig->name = bac->contig;
	    slAddTail(pContigList, contig);
	    miniIx = 0;
	    mini = NULL;
	    }
	if (mini == NULL)
	    {
	    AllocVar(mini);
	    slAddTail(&contig->miniList, mini);
	    mini->ix = ++miniIx;
	    }
	if (!inContig(contig, bac))
	    slAddTail(&mini->bacList, bac);
	}
    }
if (!gotEnd)
    warn("%s seems to be truncated\n", in->fileName);
slReverse(pContigList);
}


struct cmChrom *cmParse(char *inName)
/* Parse wash U style clone map into common
 * intermediate format. */
{
struct lineFile *in = lineFileOpen(inName, TRUE);
int lineSize;
char *line;
char *words[16];
int wordCount;
char chromName[32];
char lastChromName[32];
boolean isOrdered;
char *s;
struct cmChrom *chromList = NULL, *chrom;

strcpy(lastChromName, "");
while (lineFileNext(in, &line, &lineSize))
    {
    struct cmContig **pContigList;

    if (!startsWith("start human SUPERLINK", line))
	errAbort("Expecting start human line %d of %s\n", in->lineIx, in->fileName);
    wordCount = chopLine(line, words);
    if (wordCount != 5)
	errAbort("Odd start line %d of %s\n", in->lineIx, in->fileName);
    s = strrchr(words[2], '.');
    if (s == NULL)
	errAbort("Couldn't find chromosome line %d of 5s\n", in->lineIx, in->fileName);
    s += 1;
    strncpy(chromName, s, sizeof(chromName));
    if (!sameString(chromName, lastChromName))
	{
	strcpy(lastChromName, chromName);
	printf("Reading %s\n", chromName);
	AllocVar(chrom);
	chrom->name = cloneString(chromName);
	slAddHead(&chromList, chrom);
	}
    isOrdered = sameWord(words[3], "ORDERED");
    pContigList = (isOrdered ? &chrom->orderedList : &chrom->randomList);
    readContigList(in, pContigList);
    }
slReverse(&chromList);
return chromList;
}

boolean wannaMakeDir(char *dir)
/* Make dir if possible.  Return FALSE if not. */
{
if (mkdir(dir, 0775) < 0)
    {
    warn("\nCouldn't make directory %s", dir);
    fprintf(errLog, "%s exists\n", dir);
    perror("");
    return FALSE;
    }
return TRUE;
}

void mustMakeDir(char *dir)
/* Make directory or die trying.  Error even if dir exists. */
{
if (!wannaMakeDir(dir))
    {
    noWarnAbort();
    }
}

void mustChangeDir(char *dir)
/* Change directory or die trying. */
{
if (chdir(dir) < 0)
    {
    warn("Couldn't change directory to %s", dir);
    perror("");
    noWarnAbort();
    }
}

void makeCmDir(struct cmChrom *chromList, struct hash *finHash, 
	struct hash *predraftHash, char *outDir)
/* Make directory full of goodies corresponding to chromList. */
{
struct cmChrom *chrom;
struct cmContig *contig;
struct cmMini *mini;
struct cmBac *bac;
int isOrdered;
FILE *cinfo;
FILE *full;
FILE *geno;

mustMakeDir(outDir);
mustChangeDir(outDir);
for (chrom = chromList; chrom != NULL; chrom = chrom->next)
    {
    printf("Writing chromosome %s", chrom->name);
    fflush(stdout);
    mustMakeDir(chrom->name);
    mustChangeDir(chrom->name);
    for (isOrdered = 1; isOrdered >= 0; --isOrdered)
	{
	contig = (isOrdered ? chrom->orderedList : chrom->randomList);
	for ( ; contig != NULL; contig = contig->next)
	    {
	    printf(".");
	    fflush(stdout);
	    if (wannaMakeDir(contig->name))
		{
		mustChangeDir(contig->name);
		cinfo = mustOpen("info", "w");
		full = mustOpen("geno.lst", "w");
		fprintf(cinfo, "%s %s\n", 
		    contig->name, 
		    (isOrdered ? "ORDERED" : "RANDOM"),
		    slCount(contig->miniList));

		for (mini = contig->miniList; mini != NULL; mini = mini->next)
		    {
		    for (bac = mini->bacList; bac != NULL; bac = bac->next)
			{
			char *dir;
			if (hashLookup(finHash, bac->acc))
			    dir = finDir;
			else if (hashLookup(predraftHash, bac->acc))
			    dir = predraftDir;
			else
			    dir = unfinDir;
			fprintf(cinfo, " %s %d %d\n", bac->acc, bac->pos, bac->phase);
			fprintf(full, "%s/%s.fa\n", dir, bac->acc);
			}
		    }
		fclose(cinfo);
		fclose(full);
		mustChangeDir("..");
		}
	    }
	}


    mustChangeDir("..");
    printf("\n");
    }
}

void accHash(char *dir, struct hash *accHash)
/* Read in dir and store file names in hash keyed by
 * accession names. */
{
struct slName *list, *el;

list = listDir(dir, "*.fa");
for (el = list; el != NULL; el = el->next)
    {
    char rootFile[256];
    char *s;
    strcpy(rootFile, el->name);
    s = strrchr(rootFile, '.');
    if (s != NULL) *s = 0;	/* remove suffix. */
    hashAdd(accHash, rootFile, NULL);
    }
}

void cmProcess(char *inName, char *outDir)
/* Read in file and create corresponding dir full of goodies. */
{
struct cmChrom *chromList = cmParse(inName);
struct hash *finHash = newHash(0);
struct hash *predraftHash = newHash(0);

if (chromList == NULL)
    errAbort("Nothing to do in %s\n", inName);
printf("Reading fin dir...\n");
accHash(finDir, finHash);
printf("Reading predraft dir...\n");
accHash(predraftDir, predraftHash);
makeCmDir(chromList, finHash, predraftHash, outDir);
}

int main(int argc, char *argv[])
/* Process command line. */
{
struct cmChrom *chromList = NULL, *chrom;
if (argc != 3)
    usage();
errLog = mustOpen("err.log", "w");
cmProcess(argv[1], argv[2]);
}



